.performance-bar {
  position: relative;
  top: 2px;
  display: inline-block;
  width: 75px;
  height: 10px;
  margin: 0 0 0 5px;
  list-style: none;
  background-color: rgba(0, 0, 0, .5);
  border: 1px solid rgba(0, 0, 0, .7);
  border-radius: 2px;
  box-shadow: 0 1px 0 rgba(255, 255, 255, .15);

  li {
    position: absolute;
    top: 0;
    bottom: 0;
    overflow: hidden;
    opacity: .8;
    color: transparent;

    &:hover {
      opacity: 1;
      cursor: default;
    }
  }
}
